# EZRestrict by David Bowland
# ./addons/eventscripts/ezrestrict/ezrestrict.py

# >>> To configure this addon please see ezrestrict.cfg (created when the addon is first loaded) <<<

from __future__ import with_statement

import cfglib
import cmdlib
import es
import gamethread
import langlib
import playerlib
import usermsg
import weaponlib

from itertools import *
from path import path

import psyco
psyco.full()


info = es.AddonInfo()
info.name     = 'EZRestrict'
info.version  = '8'
info.basename = 'ezrestrict'
info.url      = 'http://addons.eventscripts.com/addons/view/' + info.basename
info.author   = 'SuperDave'

addonpath = path(__file__).dirname()

###

config = cfglib.AddonCFG(addonpath.joinpath(info.basename + '.cfg'))

config.text(info.name + ' release %s options' % info.version)
config.text('./addons/eventscripts/%(basename)s/%(basename)s.cfg' % {'basename': info.basename})
config.text(info.url)
config.text('')
config.text('Load this addon with: es_load ' + info.basename)
config.text('\n')
config.text('To configure language settings for %s please see %s_languages.ini' % (info.name, info.basename))
config.text('\n')
config.text('>>> THIS ADDON DOES NOTHING UNLESS PROMPTED <<<')
config.text('PLEASE NOTE *PLAYER* restrictions only last until the end of the map. PLAYER restrictions must be set again for each map.')
config.text('@all, @t, and @ct restrictions never expire.')
config.text('\n')

config.text('***** Options *****')
cvar_restrict_repickup     = config.cvar('ezrestrict_restrict_repickup',     2, 'Number of seconds a weapon will be unavaible for pickup after a restricted player attempts to pick the weapon up')
cvar_restrict_removebanned = config.cvar('ezrestrict_restrict_removebanned', 1, '0 = no change, 1 = when a player picks up a weapon both teams are restricted from the weapon is removed')
cvar_announce              = config.cvar('ezrestrict_announce',              0, '0 = no change, 1 = players receive a console message when they try to pick up a restricted weapon, 2 = players receive a chat area message when they try to pick up a restricted weapon')
config.text('\n')

config.text('***** Usage documentation *****')
config.text('')
config.text('The new server commands introduced by this addon are:')
config.text('')
config.text('ezrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]')
config.text('')
config.text('ezunrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]')
config.text('')
config.text('ezrestrict_removeidle [weapon 1] [weapon 2] ... [weapon n]')
config.text('*** When used without arguments ezrestrict_removeidle removes all idle weapons (excluding the C4)')
config.text('')
config.text('ezrestrict_saferemove <index>')
config.text('*** Ensures the entity exists before removing it (removing a non-existent entity will crash the server)')
config.text('')
config.text('You can use these commands in an addon, with rcon, or directly on the server.')
config.text('\n')
config.text('*** The <player/team> argument accepts: userid / player name / SteamID / #t / @t / #ct / @ct / #all / @all')
config.text('')
config.text('@ arguments apply to teams directly. # arguments apply to members of the team.')
config.text("Players will be prohibited from picking up a weapon if the weapon is restricted either to the player or the player's team.")
config.text('')
config.text('*** The <weapon> argument accepts: weapon name / #all / #primary / #rifle / #shotgun / #smg / #sniper / #secondary / #pistol / #knife / #grenade')
config.text('    In you can exclude # categories by replacing the "#" with a "!", ex: #rifle,!sniper')
config.text('    For a complete list of tags available for every game see: http://python.eventscripts.com/pages/Weaponlib')
config.text('')
config.text('Below are acceptable weapon names:')
config.text('')
config.text('deagle       - Night Hawk .50C')
config.text('ak47         - CV-47')
config.text('scout        - Schmidt Scout')
config.text('aug          - Bullpup')
config.text('g3sg1        - D3/AU1')
config.text('galil        - IDF Defender')
config.text('famas        - Clarion 5.56')
config.text('m4a1         - Maverick M4A1 Carbine')
config.text('sg552        - Krieg 552')
config.text('sg550        - Krieg 550 Commando')
config.text('m249         - M249')
config.text('awp          - Magnum Sniper Rifle')
config.text('tmp          - Schmidt Machine Pistol')
config.text('mp5navy      - KM Sub-Machine Gun')
config.text('glock        - 9x19mm Sidearm')
config.text('elite        - .40 Dual Elites')
config.text('m3           - Leone 12 Gauge Super')
config.text('xm1014       - Leone YG1265 Auto Shotgun')
config.text('usp          - KM .45 Tactical')
config.text('mac10        - Ingram MAC-10')
config.text('ump45        - KM UMP45')
config.text('p228         - 228 Compact')
config.text('fiveseven    - ES Five-Seven')
config.text('p90          - ES C90')
config.text('hegrenade    - HE Grenade')
config.text('flashbang    - Flashbang')
config.text('smokegrenade - Smoke Grenade')
config.text('knife        - Knife')
config.text('c4           - C4 (Note that C4 is not included in the #all tag)')
config.text('')
config.text('Weapon names can also be preceded with "weapon_"')
config.text('')
config.text('*** The <index> argument accepts only an entity index number.')
config.text('\n')

config.text('***** Event documentation *****')
config.text('')
config.text('This addon creates the following event which can be utilized by other addons similar to standard events:')
config.text('')
config.text('ezrestricted_pickup')
config.text('   event_var(userid)          - Userid of the player who picked up the restricted weapon')
config.text('   event_var(weapon)          - Restricted weapon that was picked up')
config.text('   event_var(longname)        - In-game name of the restricted weapon that was picked up')
config.text('   event_var(weaponindex)     - Index of the restricted weapon that was picked up (returns -1 if the weapon was removed)')
config.text('')
config.text('This event also provides event_var(es_steamid), event_var(es_username), and all the other event_vars that accompany')
config.text('event_var(userid). More information can be found here: http://www.eventscripts.com/pages/Extended_event_variables')
config.text('\n')

config.text('***** Python documentation *****')
config.text('')
config.text('In addition to server commands EZRestrict offers Python commands. In order to utilize the Python commands EZRestrict must be loaded')
config.text('before your addon and your addon must contain the import line:')
config.text('')
config.text('ezrestrict = es.import_addon("ezrestrict")')
config.text('')
config.text('You may then use the following commands:')
config.text('')
config.text('ezrestrict.restrict(<player or players>, <weapon or weapons>)')
config.text('ezrestrict.unrestrict(<player or players>, <weapon or weapons>)')
config.text('ezrestrict.removeidle([weapon or weapons])')
config.text('ezrestrict.saferemove(<index>)')
config.text('')
config.text('Player arguments can be an integer userid, a string containing a value acceptable as a player argument to the server commands, or any object that can be iterated containing acceptable string or integer values.')
config.text('Weapon arguments can be a string or any object that can be iterated containing values acceptable as weapon arguments to the server commands.')
config.text('Index argument is an integer entity index or any object that can be iterated containing integer entity indexes.')

config.write()

###

ini = cfglib.AddonINI(addonpath.joinpath(info.basename + '_languages.ini'))

ini.setInitialComments([
 '# ./addons/eventscripts/%(basename)s/%(basename)s_languages.ini' % {'basename': info.basename},
 '# >>> For general addon configuration see ' + info.basename + '.cfg <<<',
 '',
 '# DO NOT translate words with $ or # in front!',
 '# For a list of language abbreviations please see ./addons/eventscripts/_libs/python/deflangs.ini'])

ini.addGroup('restricted pickup')
ini.addValueToGroup('restricted pickup', 'en', '#defaultYou are restricted from the #lightgreen$weapon#default.')

ini.addGroup('unknown weapon')
ini.addValueToGroup('unknown weapon', 'en', 'Unknown (unknown)')

ini.addGroup('deagle')
ini.addValueToGroup('deagle', 'en', 'Night Hawk .50c (deagle)')

ini.addGroup('ak47')
ini.addValueToGroup('ak47', 'en', 'CV-47 (ak47)')

ini.addGroup('scout')
ini.addValueToGroup('scout', 'en', 'Schmidt scout (scout)')

ini.addGroup('aug')
ini.addValueToGroup('aug', 'en', 'Bullpup (aug)')

ini.addGroup('g3sg1')
ini.addValueToGroup('g3sg1', 'en', 'D3/AU-1 (g3sg1)')

ini.addGroup('galil')
ini.addValueToGroup('galil', 'en', 'IDF defender (galil)')

ini.addGroup('famas')
ini.addValueToGroup('famas', 'en', 'Clarion 5.56 (famas)')

ini.addGroup('m4a1')
ini.addValueToGroup('m4a1', 'en', 'Maverick M4A1 carbine (m4a1)')

ini.addGroup('sg552')
ini.addValueToGroup('sg552', 'en', 'Krieg 552 (sg552)')

ini.addGroup('sg550')
ini.addValueToGroup('sg550', 'en', 'Krieg 550 commando (sg550)')

ini.addGroup('m249')
ini.addValueToGroup('m249', 'en', 'M249 (m249)')

ini.addGroup('awp')
ini.addValueToGroup('awp', 'en', 'Magnum sniper rifle (awp)')

ini.addGroup('tmp')
ini.addValueToGroup('tmp', 'en', 'Schmidt machine pistol (tmp)')

ini.addGroup('mp5navy')
ini.addValueToGroup('mp5navy', 'en', 'KM sub-machine gun (mp5navy)')

ini.addGroup('glock')
ini.addValueToGroup('glock', 'en', '9x19mm sidearm (glock)')

ini.addGroup('elite')
ini.addValueToGroup('elite', 'en', '.40 dual elites (elite)')

ini.addGroup('m3')
ini.addValueToGroup('m3', 'en', 'Leone 12 gauge shotgun (m3)')

ini.addGroup('xm1014')
ini.addValueToGroup('xm1014', 'en', 'Leone YG1265 auto shotgun (xm1014)')

ini.addGroup('mac10')
ini.addValueToGroup('mac10', 'en', 'Ingram MAC-10 (mac10)')

ini.addGroup('ump45')
ini.addValueToGroup('ump45', 'en', 'KM UMP45 (ump45)')

ini.addGroup('usp')
ini.addValueToGroup('usp', 'en', 'KM .45 tactical (usp)')

ini.addGroup('p228')
ini.addValueToGroup('p228', 'en', '228 compact (p228)')

ini.addGroup('fiveseven')
ini.addValueToGroup('fiveseven', 'en', 'ES five-seven (fiveseven)')

ini.addGroup('p90')
ini.addValueToGroup('p90', 'en', 'ES C90 (p90)')

ini.addGroup('hegrenade')
ini.addValueToGroup('hegrenade', 'en', 'HE grenade (hegrenade)')

ini.addGroup('smokegrenade')
ini.addValueToGroup('smokegrenade', 'en', 'Smoke grenade (smokegrenade)')

ini.addGroup('flashbang')
ini.addValueToGroup('flashbang', 'en', 'Flashbang (flashbang)')

ini.addGroup('knife')
ini.addValueToGroup('knife', 'en', 'Knife (knife)')

ini.addGroup('c4')
ini.addValueToGroup('c4', 'en', 'C4 (c4)')

ini.addGroup('30cal')
ini.addValueToGroup('30cal', 'en', '30 Caliber Machine Gun (30cal)')

ini.addGroup('amerhand')
ini.addValueToGroup('amerhand', 'en', 'American Knife (amerhand)')

ini.addGroup('bar')
ini.addValueToGroup('bar', 'en', 'Browning Automatic Rifle (bar)')

ini.addGroup('bazooka')
ini.addValueToGroup('bazooka', 'en', 'Bazooka (bazooka)')

ini.addGroup('basebomb')
ini.addValueToGroup('basebomb', 'en', 'Bomb Satchel (basebomb)')

ini.addGroup('c96')
ini.addValueToGroup('c96', 'en', 'C96 Auto Pistol (c96)')

ini.addGroup('colt')
ini.addValueToGroup('colt', 'en', '.45 ACP 1911 (colt)')

ini.addGroup('frag_ger')
ini.addValueToGroup('frag_ger', 'en', 'German Frag Grenade (frag_ger)')

ini.addGroup('frag_us')
ini.addValueToGroup('frag_us', 'en', 'American Frag Grenade (frag_us)')

ini.addGroup('garand')
ini.addValueToGroup('garand', 'en', 'M1 Garand (garand)')

ini.addGroup('k98')
ini.addValueToGroup('k98', 'en', 'k98 Mauser (k98)')

ini.addGroup('k98s')
ini.addValueToGroup('k98s', 'en', 'k98 Mauser Sniper (k98s)')

ini.addGroup('m1carb')
ini.addValueToGroup('m1carb', 'en', 'M1 Carbine (m1carb)')

ini.addGroup('mg42')
ini.addValueToGroup('mg42', 'en', 'MG-42 (mg42)')

ini.addGroup('mp40')
ini.addValueToGroup('mp40', 'en', 'MP 40 (mp40)')

ini.addGroup('mp44')
ini.addValueToGroup('mp44', 'en', 'MP 44 (mp44)')

ini.addGroup('p38')
ini.addValueToGroup('p38', 'en', 'P38 Luger (p38)')

ini.addGroup('punch_ger')
ini.addValueToGroup('punch_ger', 'en', 'Fist (punch_ger)')

ini.addGroup('punch_us')
ini.addValueToGroup('punch_us', 'en', 'Fist (punch_us)')

ini.addGroup('pschreck')
ini.addValueToGroup('pschreck', 'en', 'Panzerschreck (pschreck)')

ini.addGroup('riflegren_ger')
ini.addValueToGroup('riflegren_ger', 'en', 'German Rifle Grenade (riflegren_ger)')

ini.addGroup('riflegren_us')
ini.addValueToGroup('riflegren_us', 'en', 'American Rifle Grenade (riflegren_us)')

ini.addGroup('smoke_ger')
ini.addValueToGroup('smoke_ger', 'en', 'German Smoke Grenade (riflegren_ger)')

ini.addGroup('smoke_us')
ini.addValueToGroup('smoke_us', 'en', 'American Smoke Grenade (riflegren_us)')

ini.addGroup('spade')
ini.addValueToGroup('spade', 'en', 'German Spade (spade)')

ini.addGroup('spring')
ini.addValueToGroup('spring', 'en', '1903 Sniper Rifle (spring)')

ini.addGroup('stick')
ini.addValueToGroup('stick', 'en', 'German Stick Grenade (stick)')

ini.addGroup('thompson')
ini.addValueToGroup('thompson', 'en', 'Thompson SMG (thompson)')

ini.addGroup('357')
ini.addValueToGroup('357', 'en', '.357 Magnum (357)')

ini.addGroup('alyxgun')
ini.addValueToGroup('alyxgun', 'en', ' (alyxgun)')

ini.addGroup('annabelle')
ini.addValueToGroup('annabelle', 'en', ' (annabelle)')

ini.addGroup('ar2')
ini.addValueToGroup('ar2', 'en', 'Overwatch Standard Issue (ar2)')

ini.addGroup('brickbat')
ini.addValueToGroup('brickbat', 'en', ' (brickbat)')

ini.addGroup('bugbait')
ini.addValueToGroup('bugbait', 'en', ' (bugbait)')

ini.addGroup('citizenpackage')
ini.addValueToGroup('citizenpackage', 'en', ' (citizenpackage)')

ini.addGroup('citizensuitcase')
ini.addValueToGroup('citizensuitcase', 'en', ' (citizensuitcase)')

ini.addGroup('crossbow')
ini.addValueToGroup('crossbow', 'en', 'Crossbow (crossbow)')

ini.addGroup('crowbar')
ini.addValueToGroup('crowbar', 'en', 'Crowbar (crowbar)')

ini.addGroup('extinguisher')
ini.addValueToGroup('extinguisher', 'en', ' (extinguisher)')

ini.addGroup('frag')
ini.addValueToGroup('frag', 'en', 'Grenade (frag)')

ini.addGroup('physcannon')
ini.addValueToGroup('physcannon', 'en', 'Gravity Gun (physcannon)')

ini.addGroup('physgun')
ini.addValueToGroup('physgun', 'en', ' (physgun)')

ini.addGroup('pistol')
ini.addValueToGroup('pistol', 'en', '9mm Pistol (pistol)')

ini.addGroup('rpg')
ini.addValueToGroup('rpg', 'en', 'RPG (rpg)')

ini.addGroup('shotgun')
ini.addValueToGroup('shotgun', 'en', 'Shotgun (shotgun)')

ini.addGroup('slam')
ini.addValueToGroup('slam', 'en', 'S.L.A.M. (slam)')

ini.addGroup('smg1')
ini.addValueToGroup('smg1', 'en', 'SMG (smg1)')

ini.addGroup('stunstick')
ini.addValueToGroup('stunstick', 'en', 'Stunstick (stunstick)')

ini.write()

lang_text = langlib.Strings(ini)


def remove_tags(text):
   """ Removes #lightgreen, #darkgreen, #green and #default tags from the supplied string """
   return reduce(lambda t, r: t.replace(r, ''), ('#lightgreen', '#darkgreen', '#green', '#default'), text)

###

class EventContext(object):
   def __init__(self, name):
      self.name = name

   def __enter__(self):
      es.event('initialize', self.name)
      return self

   def __exit__(self, type_, value, traceback_):
      es.event('fire', self.name)
      # Not returning True causes any exceptions to be re-raised

   def set(self, type_, item, value):
      es.event('set' + type_, self.name, item, value)

###

class BaseRestriction(object):
   def __init__(self):
      self.restrictions = set()

   def add(self, itr):
      self.restrictions.update(imap(weaponlib.getWeapon, makeIterable(itr)))

   def remove(self, itr):
      self.restrictions.difference_update(imap(weaponlib.getWeapon, makeIterable(itr)))

   def isRestricted(self, weapon):
      weapon = weaponlib.getWeapon(weapon)
      if weapon is None:
         return False
      return weapon in self.restrictions

def makeIterable(itr):
   if isinstance(itr, basestring) or not hasattr(itr, '__iter__'):
      return (itr,)
   return itr

###

class AllRestriction(BaseRestriction):
   def add(self, itr):
      super(AllRestriction, self).add(itr)
      for userid in es.getUseridList():
         players[userid].updateRestrictions()

   def clear(self):
      self.restrictions.clear()

allplayers = AllRestriction()

###

class TeamRestriction(BaseRestriction):
   def __init__(self, team):
      self.team = team
      super(TeamRestriction, self).__init__()

   def add(self, itr):
      def isThisTeam(userid):
         return es.getplayerteam(userid) == team
      super(TeamRestriction, self).add(itr)
      team = self.team
      for userid in ifilter(isThisTeam, es.getUseridList()):
         players[userid].updateRestrictions()

   def isRestricted(self, weapon):
      return super(TeamRestriction, self).isRestricted(weapon) or allplayers.isRestricted(weapon)

#

class TeamRestrictionDict(dict):
   def __getitem__(self, team):
      team = int(team)
      if team in self:
         return super(TeamRestrictionDict, self).__getitem__(team)
      x = self[team] = TeamRestriction(team)
      return x

teams = TeamRestrictionDict()

###

class PlayerRestriction(BaseRestriction):
   def __init__(self, userid):
      self.userid = userid
      super(PlayerRestriction, self).__init__()

   def add(self, itr):
      super(PlayerRestriction, self).add(itr)
      self.updateRestrictions()

   def isRestricted(self, weapon):
      return (super(PlayerRestriction, self).isRestricted(weapon) or
       teams[es.getplayerteam(self.userid)].isRestricted(weapon) or
       allplayers.isRestricted(weapon))

   def updateRestrictions(self):
      # Check the player's current weapons to see if any are restricted
      userid = self.userid
      for x in xrange(48):
         index = es.getindexfromhandle(es.getplayerprop(userid, 'CBaseCombatCharacter.m_hMyWeapons.%03d' % x))
         if index:
            classname = es.entitygetvalue(index, 'classname')
            if self.isRestricted(classname):
               self.removeWeapon(classname, index)

   def itemPickup(self, item):
      if self.isRestricted(item):
         weapon = str(weaponlib.getWeapon(item))
         index = self.removeWeapon(weapon)
         self.announceRestrictedPickup(weapon, index)

   def announceRestrictedPickup(self, weapon, index=None):
      weapon_text = weapon_name = weapon.split('weapon_')[~0]
      if cvar_announce == 2:
         lang = playerlib.getPlayer(self.userid).getLanguage()
         if weapon_name in lang_text:
            weapon_text = lang_text(weapon_name, lang)
         es.tell(self.userid, '#multi',
          lang_text('restricted pickup', {'weapon': weapon_text}, lang))
      elif cvar_announce:
         lang = playerlib.getPlayer(self.userid).getLanguage()
         if weapon_name in lang_text:
            weapon_text = lang_text(weapon_name, lang)
         usermsg.echo(self.userid, remove_tags(
          lang_text('restricted pickup', {'weapon': weapon_text}, lang)))

      # Fire the "ezrestricted_pickup" event
      with EventContext('ezrestricted_pickup') as event:
         event.set('int', 'userid', self.userid)
         event.set('string', 'weapon', weapon)
         event.set('string', 'longname', lang_text(weapon_name if weapon_name in lang_text else 'unknown weapon'))
         event.set('int', 'weaponindex', index or -1)

   def removeWeapon(self, classname, index=None):
      userid = self.userid
      if index is None: # Find the weapon's index if we need to
         handle = es.getplayerhandle(userid)
         for x in es.createentityindexlist(classname):
            if es.getindexprop(x, 'CBaseEntity.m_hOwnerEntity') == handle:
               index = x
               break
         else:
            raise ValueError, 'Unable to find "' + classname + '" belonging to ' + es.getplayername(userid)

      # Remove the old weapon
      if es.createplayerlist(userid)[userid]['weapon'] == str(classname):
         es.cexec(userid, 'lastinv')
      es.entitysetvalue(index, 'targetname', 'ezrestrict_remove')
      es.fire(userid, 'ezrestrict_remove', 'Kill')

      # Recreate the restricted weapon at the player's feet
      if cvar_restrict_removebanned and teams[2].isRestricted(classname) and teams[3].isRestricted(classname):
         return None
      index = es.createentity(classname)
      es.entitysetvalue(index, 'origin', es.getplayerprop(userid, 'CBaseEntity.m_vecOrigin').replace(',', ' '))
      es.entitysetvalue(index, 'spawnflags', 2)
      es.spawnentity(index)
      gamethread.delayedname(cvar_restrict_repickup, 'ezrestict', self.setSpawnFlag, (index, classname))
      return index

   @staticmethod
   def setSpawnFlag(index, classname):
      entlist = es.createentitylist(index)
      if entlist and entlist[index]['classname'] == classname:
         es.entitysetvalue(index, 'spawnflags', 0)

#

class PlayerRestrictionDict(dict):
   def __getitem__(self, userid):
      userid = int(userid)
      if userid in self:
         return super(PlayerRestrictionDict, self).__getitem__(userid)
      x = self[userid] = PlayerRestriction(userid)
      return x

   def __delitem__(self, userid):
      userid = int(userid)
      if userid in self:
         super(PlayerRestrictionDict, self).__delitem__(userid)

players = PlayerRestrictionDict()

###

def getTargets(targets):
   targets = makeIterable(targets)
   final_targets = set()
   for t in targets:
      if isinstance(t, basestring) and t.startswith(('#', '!')):
         final_targets.update(imap(players.__getitem__, playerlib.getUseridList(t)))
      elif isinstance(t, basestring) and t.startswith('@'):
         if t == '@all':
            final_targets.add(allplayers)
         elif t == '@t':
            final_targets.add(teams[2])
         elif t == '@ct':
            final_targets.add(teams[3])
      else:
         userid = es.getuserid(t)
         if userid:
            final_targets.add(players[userid])
   return final_targets


def getWeapons(weapons):
   weapons = makeIterable(weapons)
   final_weapons = set()
   for w in weapons:
      if isinstance(w, basestring) and w.startswith(('#', '!')):
         final_weapons.update(weaponlib.getWeaponList(w))
      else:
         weapon = weaponlib.getWeapon(w)
         if not weapon is None:
            final_weapons.add(weapon)
   if str(weapons[0]) == '#all': # Backwards-compatibility, remove objectives from "#all"
      final_weapons.difference_update(weaponlib.getWeaponList('#objective'))
   return final_weapons


def restrict(targets, weapons):
   weapons = getWeapons(weapons)
   for target in getTargets(targets):
      target.add(weapons)


def unrestrict(targets, weapons):
   weapons = getWeapons(weapons)
   for target in getTargets(targets):
      target.remove(weapons)


def removeidle(weapons):
   userid = es.getuserid()
   if not userid:
      return
   for weapon in getWeapons(weapons):
      for index in weapon.indexlist:
         if es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') == -1:
            es.entitysetvalue(index, 'targetname', 'ezrestrict_remove')
   es.fire(userid, 'ezrestrict_remove', 'Kill')


def saferemove(indexes):
   for index in makeIterable(indexes):
      if es.createentityindexlist(index):
         # Even if we find no userid it is likely EZRestrict will remove other weapons in the future
         # If you don't buy that, remember no players = nobody to interact with the removed entity
         es.entitysetvalue(index, 'targetname', 'ezrestrict_remove')
   userid = es.getuserid()
   if userid:
      es.fire(userid, 'ezrestrict_remove', 'Kill')

#

def ezrestrict_cmd(args):
   if len(args) >= 2:
      restrict(args[0], args[1:])
   else:
      es.dbgmsg(0, 'Syntax: ezrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]')

cmdlib.registerServerCommand('ezrestrict', ezrestrict_cmd,
 'Syntax: ezrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]')


def ezunrestrict_cmd(args):
   if len(args) >= 2:
      unrestrict(args[0], args[1:])
   else:
      es.dbgmsg(0, 'Syntax: ezunrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]')

cmdlib.registerServerCommand('ezunrestrict', ezunrestrict_cmd,
 'Syntax: ezunrestrict <player/team> <weapon 1> [weapon 2] ... [weapon n]')


def removeidle_cmd(args):
   removeidle(args or '#all')

cmdlib.registerServerCommand('ezrestrict_removeidle', removeidle_cmd,
 'Syntax: ezrestrict_removeidle [weapon 1] [weapon 2] ... [weapon n]')


def saferemove_cmd(args):
   if len(args) == 1:
      saferemove(int(args[0]))
   else:
      es.dbgmsg(0, 'Syntax: ezrestrict_saferemove <index>')

cmdlib.registerServerCommand('ezrestrict_saferemove', saferemove_cmd, 'Syntax: ezrestrict_saferemove <index>')

###

def load_events():
   if addonpath.joinpath(info.basename + '.res').isfile():
      es.loadevents('declare', 'addons/eventscripts/%(basename)s/%(basename)s.res' % {'basename': info.basename})

   else:
      events = {'ezrestricted_pickup': {
       'userid': 'short',
       'weapon': 'string',
       'longname': 'string',
       'weaponindex': 'short'}}

      if not addonpath.joinpath('es_' + info.basename + '_events_db.txt').isfile():
         es.keygroupcreate(info.basename + '_events')
         for event_name in events:
            es.keycreate(info.basename + '_events', event_name)
            for event_var in events[event_name]:
               es.keysetvalue(info.basename + '_events', event_name, event_var, events[event_name][event_var])
         es.keygroupsave(info.basename + '_events', '|' + info.basename)
         es.keygroupdelete(info.basename + '_events')
      es.loadevents('declare', 'addons/eventscripts/' + info.basename + '/es_' + info.basename + '_events_db.txt')
      es.dbgmsg(0, info.name + ': Unable to load ' + info.basename + '.res! Please ensure it is in the ./' + info.basename + '/ directory.')

load_events()

###

def clientCommandFilter(userid, args):
   if args[0].lower() == 'buy' and len(args) == 2 and players[userid].isRestricted(args[1]):
      players[userid].announceRestrictedPickup(str(weaponlib.getWeapon(args[1])))
      return False
   return True

es.addons.registerClientCommandFilter(clientCommandFilter)

###

def es_map_start(event_var):
   load_events()
   players.clear()


def round_start(event_var):
   gamethread.cancelDelayed('ezrestrict')


def dod_round_start(event_var):
   gamethread.cancelDelayed('ezrestrict')


def item_pickup(event_var):
   players[event_var['userid']].itemPickup(event_var['item'])


def player_disconnect(event_var):
   del players[event_var['userid']]


def unload():
   es.addons.unregisterClientCommandFilter(clientCommandFilter)

   cmdlib.unregisterServerCommand('ezrestrict')
   cmdlib.unregisterServerCommand('ezunrestrict')
   cmdlib.unregisterServerCommand('ezrestrict_removeidle')
   cmdlib.unregisterServerCommand('ezrestrict_saferemove')

   userid = es.getuserid()
   if userid: # Reset all weapon spawn flags on unload
      es.fire(userid, 'weapon_*', 'addoutput', 'spawnflags 0')
   gamethread.cancelDelayed('ezrestrict')